require "import"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "android.media.AudioManager"
import "android.speech.tts.UtteranceProgressListener"
import "android.speech.tts.TextToSpeech"
import "java.io.File"
import "java.io.FileOutputStream"
import "java.io.OutputStreamWriter"
import "java.io.BufferedReader"
import "java.io.InputStreamReader"
import "java.io.FileReader"
import "java.io.FileInputStream"
import "java.io.FileWriter"
import "java.io.IOException"

context = activity or service

function hasInternetConnection()
    local connectivityManager = this.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

function kill()
    speechRecognizer.destroy()
    speechRecognizer = nil
    return true
end

local folderPath = "/storage/emulated/0/Grabación con Plugin"
local folder = File(folderPath)

if not folder.exists() then
    local success = folder.mkdirs()

    if not success then
        service.asyncSpeak("Não foi possível criar a pasta em: " .. folderPath)
        return false
    end
end

local counterFilePath = "/storage/emulated/0/contador.txt"
local maxChunkSize = 4000

function startListening()
    if not hasInternetConnection() then
        service.asyncSpeak("Não há conexão com a internet.O reconhecimento de fá não está disponível.")
        return false
    end

    speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
    speechListener = RecognitionListener {
        onResults = function(results)
            data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)

            if data ~= nil and data.size() > 0 then
                local recognizedText = data.get(0)
                textoIngresado = recognizedText
                textChunks = {}

                while #textoIngresado > maxChunkSize do
                    local chunkEnd = maxChunkSize

                    while string.sub(textoIngresado, chunkEnd, chunkEnd) ~= " " and chunkEnd > 1 do
                        chunkEnd = chunkEnd - 1
                    end

                    table.insert(textChunks, string.sub(textoIngresado, 1, chunkEnd))
                    textoIngresado = string.sub(textoIngresado, chunkEnd + 1)
                end

                table.insert(textChunks, textoIngresado)
                readAloud()
            end

            kill()
        end,

        onError = function()
            kill()
            startListening()
        end
    }

    recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault())
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())

    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(speechListener)

    return true
end

function readFileContent(file)
    local content = ""
    local bufferedReader = BufferedReader(InputStreamReader(FileInputStream(file)))
    local line = bufferedReader.readLine()

    while line do
        content = content .. line
        line = bufferedReader.readLine()
    end

    bufferedReader.close()
    return content
end

function createCounterFile()
    local file = File(counterFilePath)

    if not file.exists() then
        local outputStream = FileOutputStream(file)
        local writer = OutputStreamWriter(outputStream)
        writer.write("0")
        writer.close()
        outputStream.close()
    end
end

function getCounter()
    createCounterFile()
    local file = File(counterFilePath)
    local content = readFileContent(file)
    local counter = tonumber(content)
    return counter
end

function updateCounter(counter)
    local file = File(counterFilePath)
    local outputStream = FileOutputStream(file)
    local writer = OutputStreamWriter(outputStream)
    writer.write(tostring(counter))
    writer.close()
    outputStream.close()
end

function getNextRecordingNumber()
    local counter = getCounter()
    updateCounter(counter + 1)
    return counter
end

local TTS = TextToSpeech(context, function(status)
    if status == TextToSpeech.ERROR then
        service.asyncSpeak("Erro ao inicializar TextToSpeech")
    else
        -- Puedes agregar lógica adicional aquí si es necesario
    end
end)

TTS.setOnUtteranceProgressListener(UtteranceProgressListener {
    onStart = function(utteranceId)
    end,
    onDone = function(utteranceId)
        service.asyncSpeak("Fragmento " .. utteranceId:match("utteranceId(%d+)") .. " processado.")
    end,
    onError = function(utteranceId)
        service.asyncSpeak("Erro ao processar o fragmento " .. utteranceId:match("utteranceId(%d+)"))
    end
})

function readAloud()
    if TTS.isSpeaking() then
        TTS.stop()
        return
    end

    for i, textChunk in ipairs(textChunks) do
        local recordingNumber = getNextRecordingNumber()
        service.asyncSpeak("Processando fragmento " .. recordingNumber)
        local params = {
            [TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID] = "utteranceId" .. recordingNumber,
            [TextToSpeech.Engine.KEY_PARAM_STREAM] = tostring(AudioManager.STREAM_MUSIC)
        }
        local speakStatus = TTS.synthesizeToFile(textChunk, params, string.format("%s/grabacion_%d.wav", folderPath, recordingNumber))
        if speakStatus == TextToSpeech.ERROR then
            service.asyncSpeak("Erro ao chamar TTS.synthesizeToFile para el fragmento " .. recordingNumber)
        end
    end
end

function stopSpeaking()
    if TTS.isSpeaking() then
        TTS.stop()
    end
end

startListening()  -- Esto iniciará la escucha cuando se ejecute el script